import fedmsg
import fedmsg.encoding
import moksha.hub.reactor

from twisted.internet.task import LoopingCall

from . import config
from . import JobTrigger

ITEM_TYPE = 'koji_tag'

class KojiTagChangedJobTrigger(JobTrigger):
    topic = "org.fedoraproject.prod.buildsys.tag"
    jsonify = False
    config_key = 'taskotron.kojitagchangedjobtrigger.enabled'

    def __init__(self, *args, **kw):
        super(KojiTagChangedJobTrigger, self).__init__(*args, **kw)

        self.queued_tags = set()

        # check for bodhi_to_koji_tags tasks queue every config.fuse_delay seconds
        lc = LoopingCall(self.delayed_consume)
        lc.start(config.fuse_delay)

    def delayed_consume(self):
        if self.queued_tags:
            try:
                for tag in self.queued_tags:
                    # XXX ugly: do not schedule upgradepath for updates-testing-pending
                    tasks = config.koji_tag_changed_tasks
                    if tag.endswith('testing-pending'):
                        tasks.remove('upgradepath')
                    # it is sufficient to run depcheck and upgradepath only on one arch
                    # XXX hardcoding X86_64
                    self.trigger_tasks(tag, ITEM_TYPE, tasks, ['x86_64'])
            finally:
                self.queued_tags = set()
        else:
            self.log.debug("Woke up, but there were no messages.")

    def consume(self, message):
        usable_message = fedmsg.encoding.loads(message.body)

        koji_tag = usable_message['msg']['tag']

        # we want only pending tags
        if not koji_tag.endswith('pending'):
            return

        if not len(self.queued_tags):
            self.log.info('setting fuse (%d seconds) for koji_tag %s', config.fuse_delay, koji_tag)
        elif koji_tag not in self.queued_tags:
            self.log.info('adding koji_tag %s to the fuse queue', koji_tag)

        self.queued_tags.add(koji_tag)
